"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var http_1 = require("@angular/http");
var Observable_1 = require("rxjs/Observable");
require("rxjs/add/observable/of");
require("rxjs/add/operator/map");
var globalSerializedStateKey = 'HTTP_STATE_TRANSFER';
var backingStoreDIToken = 'HTTP_STATE_BACKING_STORE';
var HttpWithStateTransfer = (function () {
    function HttpWithStateTransfer(http, backingStore) {
        this.http = http;
        this.backingStore = backingStore;
    }
    HttpWithStateTransfer.prototype.stateForTransfer = function () {
        return _a = {}, _a[globalSerializedStateKey] = this.backingStore, _a;
        var _a;
    };
    HttpWithStateTransfer.prototype.get = function (url, options, requestOptions) {
        var _this = this;
        return this.getCachedResponse(/* cacheKey */ url, function () { return _this.http.get(url, requestOptions); }, options);
    };
    HttpWithStateTransfer.prototype.getCachedResponse = function (cacheKey, provider, options) {
        var _this = this;
        // By default, the cache is only used for the *first* client-side read. So, we're only performing
        // a one-time transfer of server-side response to the client. If you want to keep and reuse cached
        // responses continually during server-side and client-side execution, set 'permanent' to 'true.
        var isClient = typeof window !== 'undefined';
        var isPermanent = options && options.permanent;
        var allowReadFromCache = isClient || isPermanent;
        if (allowReadFromCache && this.backingStore.hasOwnProperty(cacheKey)) {
            var cachedValue = this.backingStore[cacheKey];
            if (!isPermanent) {
                delete this.backingStore[cacheKey];
            }
            return Observable_1.Observable.of(new http_1.Response(new http_1.ResponseOptions({
                body: cachedValue.text,
                headers: new http_1.Headers(cachedValue.headers),
                status: cachedValue.status,
                url: cachedValue.url
            })));
        }
        return provider()
            .map(function (response) {
            var allowWriteToCache = !isClient || isPermanent;
            if (allowWriteToCache) {
                _this.backingStore[cacheKey] = {
                    headers: response.headers ? response.headers.toJSON() : null,
                    status: response.status,
                    statusText: response.statusText,
                    text: response.text(),
                    url: response.url
                };
            }
            return response;
        });
    };
    /** @nocollapse */
    HttpWithStateTransfer.ctorParameters = function () { return [
        { type: http_1.Http, decorators: [{ type: core_1.Inject, args: [http_1.Http,] },] },
        { type: undefined, decorators: [{ type: core_1.Inject, args: [backingStoreDIToken,] },] },
    ]; };
    return HttpWithStateTransfer;
}());
exports.HttpWithStateTransfer = HttpWithStateTransfer;
function defaultBackingStoreFactory() {
    var transferredData = typeof window !== 'undefined' ? window[globalSerializedStateKey] : null;
    return transferredData || {};
}
exports.defaultBackingStoreFactory = defaultBackingStoreFactory;
var HttpWithStateTransferModule = (function () {
    function HttpWithStateTransferModule() {
    }
    HttpWithStateTransferModule.decorators = [
        { type: core_1.NgModule, args: [{
                    providers: [
                        // The backing store is a separate DI service so you could override exactly how it gets
                        // transferred from server to client
                        { provide: backingStoreDIToken, useFactory: defaultBackingStoreFactory },
                        { provide: HttpWithStateTransfer, useClass: HttpWithStateTransfer },
                    ]
                },] },
    ];
    /** @nocollapse */
    HttpWithStateTransferModule.ctorParameters = function () { return []; };
    return HttpWithStateTransferModule;
}());
exports.HttpWithStateTransferModule = HttpWithStateTransferModule;
